/**The MIT License (MIT)
 
 Copyright (c) 2023 by ArtSuzhou LLC
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 */

#include<ESP8266WiFi.h>
#include<espnow.h>

#define MY_NAME   "MASTER_NODE"
#define LED_PIN D4 // 14 Pin number for data received LED

// Define a global variable to store the received value and the timestamp of the last change
int receivedValue = 0;
int newValue = 0;
unsigned long lastChangeTime = 0;

void setup() {

  Serial.begin(115200);     // initialize serial port
  pinMode(LED_PIN, OUTPUT);
  
  Serial.print("Initializing...");
  Serial.println(MY_NAME);
  Serial.print("My MAC address is: ");
  Serial.println(WiFi.macAddress());

  WiFi.mode(WIFI_STA);
  WiFi.disconnect();        // we do not want to connect to a WiFi network

  if (esp_now_init() != 0) {
    Serial.println("ESP-NOW initialization failed");
    return;
  } else {
    Serial.println("Initialized.");
  };
  
  esp_now_register_recv_cb(dataReceived);   // this function will get called once all data is sent

}

// callback when data is recv from Master
void dataReceived(uint8_t *mac_addr, uint8_t *data, uint8_t data_len) {
  char macStr[18];
  snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x",
           mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
  Serial.print("Data Recv from: "); Serial.println(macStr);
  Serial.print("Received  Data: "); Serial.println(*data);
  Serial.println("");

  memcpy(&newValue, data, sizeof(int));
  Serial.print("New value: ");
  Serial.println(newValue);    

// Only update the received value and the timestamp if the value has changed
  if (newValue != receivedValue) {
      receivedValue = newValue;
      lastChangeTime = millis();
    }  
    
  if (data_len > 0) {
      digitalWrite(LED_PIN, HIGH);
     } else {
      digitalWrite(LED_PIN, LOW);
         }
}


void loop() {
 if (millis() - lastChangeTime < 1000) {
  Serial.print("Received value: ");
  Serial.println(receivedValue);   
  digitalWrite(LED_PIN, HIGH);  
  } else {
  Serial.println("No data received.");
  digitalWrite(LED_PIN, LOW);  
  }   
  delay(1000);
}
